package info.aspirecn.iov.yysj.product.check;
import java.io.IOException;

import info.aspirecn.iov.yysj.product.check.common.Property;
import info.aspirecn.iov.yysj.product.check.pre.OtherBillFilter;
import info.aspirecn.iov.yysj.product.check.pre.WayBillFilter;
import org.apache.commons.lang.math.NumberUtils;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.cloud.netflix.zuul.EnableZuulProxy;
import org.springframework.cloud.netflix.zuul.filters.ZuulProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.core.env.Environment;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.RestController;
import lombok.extern.slf4j.Slf4j;
@ComponentScan(basePackages={"info.aspirecn.iov.yysj.product"})
@SpringBootApplication
@EnableZuulProxy
@RestController
@Slf4j
@EnableCaching
public class ProductCheck {


	public static void main(String[] args) {

		StopWatch watch = new StopWatch();
		watch.start();
		ApplicationContext context = SpringApplication.run(ProductCheck.class, args);
		Environment environment = context.getBean(Environment.class);
		String applicationName = environment.getProperty("spring.application.name");
		String infoName = environment.getProperty("info.name");
		String infoVersion = environment.getProperty("info.version");
		int serverPort = NumberUtils.toInt(environment.getProperty("server.port"));
		int managementServerPort = NumberUtils.toInt(environment.getProperty("management.server.port"));
		Property property = context.getBean(Property.class);
		watch.stop();

		log.info("{} Ver.{} ({}) 启动完毕，serverPort={}, managementServerPort={}, times={}s", infoName, infoVersion,
				applicationName, serverPort, managementServerPort, watch.getTotalTimeSeconds());
	}

	/**
	 * 动态更新路由策略 经测试，可以加策略，但是删除需要重启
	 * 
	 * @return
	 */
	@RefreshScope
	@ConfigurationProperties("zuul")
	public ZuulProperties zuulProperties() {
		return new ZuulProperties();
	}
	/**
	 * 链路日志打印
	 *
	 * @throws IOException
	 */
/*	@PostMapping("/api/v2/spans")
	public void index(HttpServletRequest request) throws IOException {
		String context = IOUtils.toString(request.getInputStream(), Charsets.UTF_8);
		logger_sleuth.debug(context);
	}*/

	@Bean
	public WayBillFilter addUserBalanceFilter() {
		return new WayBillFilter();
	}

	@Bean
	public OtherBillFilter addOtherBillFilter() {
		return new OtherBillFilter();
	}

}