package info.aspirecn.iov.yysj.product.check.pre;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import info.aspirecn.cloud.yysj.commons.lang.BillItemResult;
import info.aspirecn.cloud.yysj.commons.lang.Constants;
import info.aspirecn.cloud.yysj.commons.lang.OtherCommonResult;
import info.aspirecn.cloud.yysj.commons.lang.ProductCheckRequest;
import info.aspirecn.iov.yysj.product.check.common.Property;
import info.aspirecn.iov.yysj.product.check.common.TransferParam;
import info.aspirecn.iov.yysj.product.check.common.ZuulHelper;
import info.aspirecn.iov.yysj.product.check.entity.ResponseObject;
import info.aspirecn.iov.yysj.product.check.entity.SjjhTransferReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.netflix.zuul.filters.support.FilterConstants;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * 运单信息核验服务,
 */
@Slf4j
public class OtherBillFilter extends ZuulFilter {
	@Autowired
	private ObjectMapper mapper;
	@Autowired
	private ZuulHelper helper;
	@Autowired
	private TransferParam transferParam;
	@Override
	public boolean shouldFilter() {
		HttpServletRequest request = RequestContext.getCurrentContext().getRequest();
/*		request.setAttribute("result", "0");
		request.setAttribute(Constants.price,"0");
		request.setAttribute("isCharge","0");*/
		String url = request.getRequestURI();

		return Constants.ProductCheckUrl.OTHERBILLURL.getValue().equals(url);
	}

	@Override
	public Object run() {
		String accessId = UUID.randomUUID().toString().replace("-","");
		HttpServletRequest request = RequestContext.getCurrentContext().getRequest();
		OtherCommonResult responseObject = new OtherCommonResult();
		try {
			//从请求头中获取token
			String token = request.getHeader(Constants.HEADER_TOKEN_KEY);
			int price =0;

			String body = helper.getCustomBody(request);
			ProductCheckRequest productCheckRequest = mapper.readValue(body, ProductCheckRequest.class);
			List<String> sjjhProductIds = productCheckRequest.getSjjhProductId();
			List<String> emptyList = new ArrayList<String>();

			if(sjjhProductIds.size() ==1){
				SjjhTransferReq sjjhTransferReq = new SjjhTransferReq();
				sjjhTransferReq.setAccessId(accessId);
				sjjhTransferReq.setSjjhProductId(sjjhProductIds.get(0));
				Map<String,Object> customMap =productCheckRequest.getCustomBody();
				//转换成数据交互的请求参数信息
				Object customObject = transferParam.getCustomBody(customMap,productCheckRequest.getSjjhProductId().get(0),emptyList);
				sjjhTransferReq.setCustomBody(customObject);
				//设置云验平台资格校验项校验内容，用于返回结果一致性比较
				sjjhTransferReq.setOriginCustomBody(customMap);
				sjjhTransferReq.setToken(token);

				 responseObject = helper.getOtherResult(sjjhTransferReq);
				if(responseObject!=null){
					price = responseObject.getPrice();
				}

			}

			if(Constants.inconsistent_code.equals(responseObject.getResultCode())||Constants.consistent_code.equals(responseObject.getResultCode())){
				request.setAttribute("result", responseObject.getResultCode());
			}else{
				request.setAttribute("result", Constants.error);
			}
			RequestContext.getCurrentContext().getRequest().setAttribute(Constants.price,price);
		} catch (Exception ex) {
			log.error("Exception:{}",ex);
			responseObject.setResultCode(Constants.error);
		}
		try {
			helper.responseHandleCommon(responseObject, RequestContext.getCurrentContext());
		} catch (JsonProcessingException e) {
			log.error("JsonProcessingException:{}",e);
		}
		return null;
	}

	@Override
	public String filterType() {
		return FilterConstants.PRE_TYPE;
	}

	@Override
	public int filterOrder() {
		return FilterConstants.SERVLET_30_WRAPPER_FILTER_ORDER + 4;
	}
}
