package info.aspirecn.iov.yysj.product.check.redis;

import java.nio.charset.Charset;
import java.util.Optional;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.SerializationException;
import org.springframework.scripting.support.ResourceScriptSource;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import lombok.extern.slf4j.Slf4j;
@Configuration
@Slf4j
public class RedisConfig {
	@Bean
    public RedisTemplate<String, Object> redisTemplate(RedisConnectionFactory connectionFactory) {
        RedisTemplate<String, Object> template = new RedisTemplate<>();
        template.setConnectionFactory(connectionFactory);
        //使用FastJsonRedisSerializer来序列化和反序列化redis的value值（默认使用JDK的序列化方式）
        FastJsonRedisSerializer<Object> serializer = new FastJsonRedisSerializer<Object>(Object.class);
        template.setValueSerializer(serializer);
        template.setKeySerializer(serializer);
        template.afterPropertiesSet();
        return template;
    }
	@Bean
    public DefaultRedisScript<Integer> getRedisScript() {
			DefaultRedisScript<Integer> getRedisScript = new DefaultRedisScript<Integer>();
	        getRedisScript.setResultType(Integer.class);
	        getRedisScript.setScriptSource(new ResourceScriptSource(new ClassPathResource("config/test.lua")));
	        return getRedisScript;
    }
	public class FastJsonRedisSerializer<T> implements RedisSerializer<T> {
	    public final Charset DEFAULT_CHARSET = Charset.forName("UTF-8");
	    private Class<T> clazz;
	 
	    public FastJsonRedisSerializer(Class<T> clazz){
	        super();
	        this.clazz = clazz;
	    }
	 
	    @Override
	    public byte[] serialize(T t) throws SerializationException {
	    	
	        return Optional.ofNullable(t).map(r ->  {
	        	log.debug("T:{}",t);
	        	return JSON.toJSONString(r, SerializerFeature.WriteClassName).getBytes(DEFAULT_CHARSET);
	        }).orElseGet(() -> new byte[0]);
	    }
	 
	    @Override
	    public T deserialize(byte[] bytes) throws SerializationException {
	        return Optional.ofNullable(bytes)
	                .map(t ->{ 
	                	T result = JSON.parseObject(new String(t, DEFAULT_CHARSET), clazz);
	                	log.debug("result:{}",result);
	                	return result;
	                })
	                .orElseGet(() -> null);
	    }
	 
	}
}
